#!/usr/bin/perl -w
use strict;

use lib 'lib';
use lib 'inc';

use ExtUtils::MakeMaker;
use Config '%Config';
use File::Basename 	qw[dirname basename];
use File::Path;
use File::Spec;
use Cwd             qw[cwd];

my $package = 'Alien::Iconv';
(my $filename = $package) =~ s!::!/!g;
$filename .= ".pm";
require $filename;

use vars qw[$DRY_RUN $VERBOSE];
$DRY_RUN = 0;
$VERBOSE = 0;

my @Makefile_ARGS = (
    NAME            => $package,
    VERSION_FROM    => "lib/$filename",
    dist            => { COMPRESS => 'gzip -9f', SUFFIX => 'gz' },
    AUTHOR          => 'Max Maischein <corion@cpan.org>',
	ABSTRACT        => 'Supply external iconv library',
	PREREQ_PM       => { # These are the prerequisites for Alien::Util
	                     'IPC::Cmd' => 0,
                             'File::Fetch' => 0,
                             # These are the real prerequisites
                             'Alien::Zlib' => "0.00_02",
                       },
);

WriteMakefile(@Makefile_ARGS);


my $lib;
my $cwd;
my $progs;

### if you only want to write the makefile for 'make dist'
### invoke it as 'perl Makefile.PL dist'

unless( 'dist' eq ($ARGV[0] ||"" ) ) {

    eval q[
        use IPC::Cmd;
        use File::Fetch;
    ];
    if ($@) {
        warn <<PREREQUISITES;
    
IPC::Cmd and File::Fetch are not installed. 
    
You will only be able to do

    make dist

PREREQUISITES

        exit
    };

    # Seems like we have all we need
    require Alien::Util;
    Alien::Util->import();

    my @progs = qw[tar gzip make cc];
    if ($^O ne 'MSWin32' and $^O !~ /VMS/) {
      push @progs, 'sh'; # for ./configure
    };
    $progs = Alien::Util::find_progs(@progs);

    (($lib) = $package->archives);
    $lib =~ s/\.tar\.gz$//i;
    #$lib = 'libiconv-' . $package->lib_version;
    $cwd = cwd();
    chdir $cwd;

    ### fetch the sources ###
    my @urls = $package->urls;
    my @decompress;
    for my $url (@urls) {
        my $file = Alien::Util::fetch($url);
        push @decompress, $file;
        print "Succesfully fetched $file\n";
    };
    chdir $cwd;

    ### extract them ###
    for my $src (@decompress) {
        Alien::Util::decompress($progs, $lib, $src);
    };
    chdir $cwd;

    ### patch && install ###
    install_lib();
    chdir $cwd;
}

### Write the makefile, again, this time with all the correct object files ###
WriteMakefile(@Makefile_ARGS);

### SUB ROUTINES ###

# Here, the actual build magic takes place
# This is little more than a glorified shell script
sub install_lib {
    my $dirname  = File::Spec->rel2abs(dirname($0), 'src');
    my $lib_dir  = File::Spec->catdir( $dirname, $lib );
    my $hints 	 = File::Spec->catdir( $dirname, 'hints', $^O );
    my $makefile = File::Spec->catdir( $lib_dir, 'Makefile' );
    my $prefix   = $package->prefix;
    my $include  = $package->include;
    my $lib      = $package->lib;

    #if ($^O eq 'MSWin32') {
    #    $lib_dir .= '/win32';
    #};

    #eval q{use Alien::Zlib;};

    ### go to the Win32 dir ###
    unless( chdir $lib_dir ) {
        die qq[Could not chdir to '$lib_dir' ($!) -- can not compile sources\n];
    }

    my @configure;
    my @make;
    my @install;

    if ($^O eq 'MSWin32') {

      my $perl = Alien::Util::add_quotes $^X;
      $perl =~ s!\\!\\\\!g;
      my $inc = "";#  Alien::Util::add_quotes("-I" . quotemeta Alien::Zlib->include);
      my $lib = "";#join " ", map { Alien::Util::add_quotes(quotemeta $_) } Alien::Zlib->lib_files;
      ### patch invocation for '$(RM)' to use Perl ###
      #push @configure, Alien::Util::modify( 's/-I[.][.]\\\\zlib/'.$inc.'/', $makefile );
      #push @configure, Alien::Util::modify( 's/-I[.][.]\\\\zlib\\\\zlib.lib/'.$lib.'/', $makefile );
      #push @configure, Alien::Util::modify( sprintf('s/(RM\s*=\s*)\s*del/$1%s -MExtUtils::Command -e rm_f/', $perl), $makefile );

      @make = (
		      #[ $progs->{make},	'-f', 'Makefile.msvc', 'NO_NLS=1', "PREFIX=$prefix", 'clean'		], # make a dependency
		      # If running without threads, change MFLAGS=-MD to MFLAGS=-ML
		      [ $progs->{make},	'-f', 'Makefile.msvc', 'NO_NLS=1', "PREFIX=$prefix", 'MFLAGS=-MD', 'all'		],
		      # [ $progs->{make},	'-f', 'Makefile.msvc', 'NO_NLS=1', 'check'		], # check fails when run automatically
      );
      @install = (
		      [ $progs->{make},	'-f', 'Makefile.msvc', 'NO_NLS=1', "PREFIX=$prefix", 'install'		],
      );
    } elsif ($^O =~ /VMS/) {
      # This is fudged, I'm not sure what is the distinction
      # between VMS MMS and VMS-non-MMS
      #Alien::Util::copy_file(File::Spec->catdir($lib_dir,'Makefile.vms') => File::Spec->catdir($lib_dir,$makefile));
      #Alien::Util::copy_file(File::Spec->catdir($lib_dir,'jconfig.vms')  => File::Spec->catdir($lib_dir,'jconfig.h'));
    } else {
      @configure = ( [$progs->{'sh'}, 	'configure', @ARGV],
      );
      push @make, [ $progs->{'make'},	'all'		],
      push @install, [ $progs->{'make'}, 	'install' ],
    };

    #if (glob "src//lib/*.$Config{_o}") {
    #  unshift @configure, [$progs->{'make'}, 'clean' ];
    #};

    ### build the lib sources, make sure to copy them into the lib/ dir ###
    my @cmds = (
        # Patch in CCFLAGS from %Config
        @configure,

		@make,

        @install,
    );
    warn $lib_dir;

    Alien::Util::process(@cmds);
    print "Succesfully compiled $lib\n";

    # Now, copy all we need into our lib directory
    for my $dir ($package->lib, $package->include) {
      mkpath $dir
      #  or die "Couldn't create '$dir': $!";
    };
    #for ($package->lib_files) {
    #    Alien::Util::copy_file(File::Spec->catfile($lib_dir,basename($_)) => $_)->();
    #};
    #for ($package->include_files) {
    #    Alien::Util::copy_file(File::Spec->catfile($lib_dir,basename($_)) => $_)->();
    #};
}
